//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// event.cpp
//
// event routines
//
// $Id: $
//

#include <stdio.h>
#include "typedefs.h"
#include "event.h"
#include "machine.h"


eventHandler::eventHandler(machine *pMachine)
: mpMachine(pMachine)
{
   mHead = mTail = 0;
}


void eventHandler::post(event *pEvent)
{
   mQueue[mTail] = *pEvent;

   mTail = (mTail + 1) % MAX_EVENT;
   if (mTail == mHead)
   {
      fprintf(stderr, "eventHandler::post(): mTail == mHead\n");
      mHead = (mHead + 1) % MAX_EVENT;
   }
}


int eventHandler::process()
{
   int done = 0;

   while (mHead != mTail)
   {
      event *pEvent = &mQueue[mHead];

      switch (pEvent->type)
      {
      case EVENT_NONE:
         break;
         
      case EVENT_QUIT:
         done = 1;
         break;

      case EVENT_RESET:
         mpMachine->reset(pEvent->data ? true : false);
         break;

      case EVENT_UNTHROTTLE:
         mpMachine->setThrottle(pEvent->data ? false : true);
         break;

      case EVENT_PAD0_PRESS:
         mpMachine->triggerInput(0, 1 << pEvent->data, true);
         break;

      case EVENT_PAD0_RELEASE:
         mpMachine->triggerInput(0, 1 << pEvent->data, false);
         break;

      case EVENT_PAD1_PRESS:
         mpMachine->triggerInput(1, 1 << pEvent->data, true);
         break;

      case EVENT_PAD1_RELEASE:
         mpMachine->triggerInput(1, 1 << pEvent->data, false);
         break;

      case EVENT_STATE_SAVE:
         mpMachine->saveState();
         break;
         
      case EVENT_STATE_LOAD:
         mpMachine->loadState();
         break;

      default:
         break;
      }

      mHead = (mHead + 1) % MAX_EVENT;
   }

   return done;
}
