//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// ppu.h
//
// ppu emulation
//
// $Id: $
//

#ifndef _PPU_H_
#define _PPU_H_

#include "typedefs.h"
#include "palette.h"
#include "regs.h"
#include "unit.h"

#define  NO_TILECACHE

#define  PPU_BANKSIZE      0x400
#define  PPU_NUMBANKS      16


class machine;
class oam;
class ppu : public unit
{
public:
   ppu(machine *pMachine);
   virtual ~ppu();

   virtual void reset(bool hardReset);
   
   bool  enabled() const;
   bool  vbl() const;
   int   currentLine() const;

   uint8 *scanline();

   void  mirror(int nt0, int nt1, int nt2, int nt3);
   void  bank(int size, int page, uint8 *pMemory, bool writeProtect);

   static uint8 dispatchRead(void *userdata, uint32 offset);
   static void  dispatchWrite(void *userdata, uint32 offset, uint8 value);
   static void  dispatchWriteDMA(void *userdata, uint32 offset, uint8 value);

private:
   uint8 read(uint32 offset);
   void  write(uint32 offset, uint8 value);
   void  writeDMA(uint32 offset, uint8 value);

   void  setPalette(int index, uint8 value);
   void  setStrike(int pixel);
   
   bool  vramInaccessible() const;

   void  syncHighPages();

   void  beginScanline();
   void  endScanline();
   uint8 *renderScanline();

   void  renderBGLine(uint8 *pBuffer);
   void  renderOAMLine(uint8 *pBuffer);
   void  cacheOAM();

   int   drawOAMTile(uint8 *pBuf, uint8 *pPriority, uint8 attrib,
                     uint8 pat0, uint8 pat1, uint8 colRow, bool checkStrike);

#ifndef NO_TILECACHE
   void  updateTileCache();
   void  cacheTile(int tile);
   void  cachePage(int page);

   // x4 for all possible palette combos
   uint8    mTileCache[512 * 4][8][8];
   bool     mTileDirty[512];
   bool     mAnyTilesDirty;
   bool     mPageDirty[PPU_NUMBANKS];
   bool     mAnyPagesDirty;
#endif // !NO_TILECACHE

   uint8    *mpPage[PPU_NUMBANKS];   // 1kb page pointers
   bool     mProtect[PPU_NUMBANKS];  // write-protect pages?

   // TODO: this should only be 2kb
   // find way to work around 4-screen mirror
   // mappers using 4 nametables...
   // (should be allocated on cart)

   uint8    mNameTable[0x1000];

   color    mColors[palette::NUM_PALETTE_COLORS];

   uint8    mRegs[8];

   uint32   mVAddr;
   uint32   mVAddrLatch;
   int      mTileXOfs;
   bool     mFlipFlop;

   int      mScanline;
   uint8    mLatch;
   uint8    mVDataLatch;

   uint32   mStrikeCycle;
   bool     mStrike;

   oam      *mpOAM;
   palette  mPalette;

   // 8 pixel overdraw on each side
   uint8    mScanlineBuffer[8 + 256 + 8];
};

inline bool ppu::enabled() const
{
   return (mRegs[REG_CTRL1] & (REGF_CTRL1_BGON | REGF_CTRL1_OBJON));
}

inline bool ppu::vbl() const
{
   return (mRegs[REG_CTRL0] & REGF_CTRL0_NMI);
}

inline int ppu::currentLine() const
{
   return mScanline;
}

// when the ppu is enabled, vram is available only
// from the beginning of scanline 240 to the end of 260
inline bool ppu::vramInaccessible() const
{
   //return (enabled() && !(currentLine() >= 240 && currentLine() <= 261));
   return (enabled() && currentLine() < 240);
}

#endif // !_PPU_H_

